from nose2.tests._common import FunctionalTestCase


class TestPrettyAsserts(FunctionalTestCase):
    def assertProcOutputPattern(self, proc, expected, assert_exit_status=1):
        """
        - proc: a popen proc to check output on
        - expected: expected output pattern
        """
        stdout, stderr = proc.communicate()

        if assert_exit_status is not None:
            self.assertEqual(assert_exit_status, proc.poll())

        self.assertTestRunOutputMatches(
            proc, stderr=expected)

        return stderr

    def test_simple_global(self):
        proc = self.runIn(
            'scenario/pretty_asserts/simple_global',
            '-v', '--pretty-assert',
        )
        expected = "\n".join([
            ">>> assert myglob == 2",
            "",
            "values:",
            "    myglob = 1",
            ""
        ])
        self.assertProcOutputPattern(proc, expected)

    def test_conf_on(self):
        proc = self.runIn(
            'scenario/pretty_asserts/conf_on',
            '-v',
        )
        expected = "\n".join([
            ">>> assert myglob == 2",
            "",
            "values:",
            "    myglob = 1",
            ""
        ])
        self.assertProcOutputPattern(proc, expected)

    def test_conf_on_suppresses_clihelp(self):
        proc = self.runIn(
            'scenario/pretty_asserts/conf_on',
            '--help',
        )
        stdout, stderr = proc.communicate()
        exit_status = proc.poll()
        assert '--pretty-assert' not in stdout
        assert '--pretty-assert' not in stderr
        assert exit_status == 0

    def test_conf_on_plus_arg(self):
        """ensures that #432 stays fixed"""
        proc = self.runIn(
            'scenario/pretty_asserts/conf_on',
            '-v',
            '--pretty-assert',
        )
        expected = "\n".join([
            ">>> assert myglob == 2",
            "",
            "values:",
            "    myglob = 1",
            ""
        ])
        self.assertProcOutputPattern(proc, expected)

    def test_assign_after_assert(self):
        proc = self.runIn(
            'scenario/pretty_asserts/assign_after_assert',
            '-v', '--pretty-assert',
        )
        expected = "\n".join([
            ">>> assert x == 2",
            "",
            "values:",
            "    x = 1",
            ""
        ])
        self.assertProcOutputPattern(proc, expected)

    def test_multiline_assert_statement(self):
        proc = self.runIn(
            'scenario/pretty_asserts/multiline_statement',
            '-v', '--pretty-assert',
        )
        expected = "\n".join([
            '>>> assert \\(x',
            '>>>         >',
            '>>>         y\\), \\"oh noez, x <= y\\"',
            '',
            'message:',
            '    oh noez, x <= y',
            '',
            'values:',
            '    x = 1',
            '    y = 2',
            ''
        ])
        self.assertProcOutputPattern(proc, expected)

    def test_multiline_funcdef(self):
        proc = self.runIn(
            'scenario/pretty_asserts/multiline_funcdef',
            '--pretty-assert',
        )
        expected1 = "\n".join([
            '>>> assert x > y, \\"oh noez, x <= y\\"',
            '',
            'message:',
            '    oh noez, x <= y',
            '',
            'values:',
            '    x = 1',
            '    y = 2',
            ''
        ])
        expected2 = "\n".join([
            '>>> assert not value',
            '',
            'values:',
            "    value = 'foo'",
            ''
        ])
        expected3 = "\n".join([
            '>>> assert not value',
            '',
            'values:',
            "    value = 'bar'",
            ''
        ])
        self.assertProcOutputPattern(proc, expected1)
        self.assertProcOutputPattern(proc, expected2)
        self.assertProcOutputPattern(proc, expected3)

    def test_assert_attribute_resolution(self):
        proc = self.runIn(
            'scenario/pretty_asserts/attribute_resolution',
            '-v', '--pretty-assert',
        )
        expected = "\n".join([
            ">>> assert x.x \\!= \\(self",
            ">>>                \\).x",
            "",
            "values:",
            ("    x = <test_prettyassert_attribute_resolution.TestFoo"
             " testMethod=test_ohnoez>"),
            "    x.x = 1",
            ("    self = <test_prettyassert_attribute_resolution.TestFoo"
             " testMethod=test_ohnoez>"),
            "",
        ])
        self.assertProcOutputPattern(proc, expected)

    def test_assert_attribute_resolution2(self):
        proc = self.runIn(
            'scenario/pretty_asserts/attribute_resolution2',
            '-v', '--pretty-assert',
        )
        expected = "\n".join([
            ">>> assert foo\\(\\).x \\!= self.x",
            "",
            "values:",
            ("    foo = <function (TestFoo.test_ohnoez.<locals>.|)foo at "
             "0x[a-z0-9]+>"),
            ("    self = <test_prettyassert_attribute_resolution2.TestFoo"
             " testMethod=test_ohnoez>"),
            "    self.x = 1",
            "",
        ])
        self.assertProcOutputPattern(proc, expected)

    def test_assert_ignore_passing(self):
        proc = self.runIn(
            'scenario/pretty_asserts/ignore_passing',
            '-v', '--pretty-assert',
        )
        expected1 = "\n".join([
            ">>> assert x; assert y",
            "",
            "values:",
            "    y = False",
            "",
        ])
        expected2 = "\n".join([
            ">>> assert q",
            "",
            "values:",
            "    q = 0",
            "",
        ])
        self.assertProcOutputPattern(proc, expected1)
        self.assertProcOutputPattern(proc, expected2)

    def test_unittest_assertion(self):
        proc = self.runIn(
            'scenario/pretty_asserts/unittest_assertion',
            '-v', '--pretty-assert',
        )
        # look for typical unittest output
        expected = "self.assertTrue\\(x\\)\nAssertionError: False is not true"
        stderr = self.assertProcOutputPattern(proc, expected)
        # the assertion line wasn't reprinted by prettyassert
        self.assertNotIn('>>> self.assertTrue', stderr)
        # the assertion values weren't printed by prettyassert
        self.assertNotIn('values:', stderr)



"""version information"""

# taken from http://stackoverflow.com/a/17626524/1836144
# The following line *must* be the last in the module, exactly as formatted:
# could also use advice from
# https://packaging.python.org/guides/single-sourcing-package-version/
__version__ = '0.9.1'



import sys
import os
import sphinx_rtd_theme

sys.path.insert(0, os.path.abspath('..'))
extensions = ['sphinx.ext.autodoc',
              'sphinx.ext.doctest',
              'sphinx.ext.intersphinx',
              'sphinx.ext.todo',
              'sphinx.ext.coverage',
              'sphinx.ext.ifconfig',
              'sphinx.ext.viewcode',
              'nose2.sphinxext']
source_suffix = '.rst'
master_doc = 'index'
project = u'nose2'
copyright = u'2010, Jason Pellerin'
version = '0.6'
release = '0.6.0'
exclude_patterns = ['_build']
templates_path = ['_templates']
pygments_style = 'sphinx'
html_theme = 'sphinx_rtd_theme'
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
man_pages = [
    ('index', 'nose2', u'nose2 Documentation',
     [u'Jason Pellerin'], 1)]
intersphinx_mapping = {'python': ('http://docs.python.org/', None)}



import six
from nose2 import events


class PrintFixture(events.Plugin):
    alwaysOn = True

    def startLayerSetup(self, event):
        six.print_("StartLayerSetup: {0}".format(event.layer))

    def stopLayerSetup(self, event):
        six.print_("StopLayerSetup: {0}".format(event.layer))

    def startLayerSetupTest(self, event):
        log = "StartLayerSetupTest: {0}:{1}"
        six.print_(log.format(event.layer, event.test))

    def stopLayerSetupTest(self, event):
        log = "StopLayerSetupTest: {0}:{1}"
        six.print_(log.format(event.layer, event.test))

    def startLayerTeardownTest(self, event):
        log = "StartLayerTeardownTest: {0}:{1}"
        six.print_(log.format(event.layer, event.test))

    def stopLayerTeardownTest(self, event):
        log = "StopLayerTeardownTest: {0}:{1}"
        six.print_(log.format(event.layer, event.test))

    def startLayerTeardown(self, event):
        six.print_("StartLayerTeardown: {0}".format(event.layer))

    def stopLayerTeardown(self, event):
        six.print_("StopLayerTeardown: {0}".format(event.layer))



import os
import os.path
import unittest

from nose2.tests._common import FunctionalTestCase, support_file


class TestCoverage(FunctionalTestCase):
    def assertProcOutputPattern(self, proc, libname, stats,
                                total_stats=None, assert_exit_status=0):
        """
        - proc: a popen proc to check output on
        - libname: the name of the covered library, to build the output pattern
        - stats: the output format for stats info
        - total_stats: optional, distinct stats format for the TOTAL line
                       defaults to be the same as stats
        """
        if total_stats is None:
            total_stats = stats

        expected = os.path.join(libname, 'mod1(.py)?')
        expected = expected.replace('\\', r'\\')
        expected = expected + stats

        stdout, stderr = proc.communicate()

        if assert_exit_status is not None:
            self.assertEqual(assert_exit_status, proc.poll())

        self.assertTestRunOutputMatches(
            proc,
            stderr=expected)
        self.assertTestRunOutputMatches(
            proc,
            stderr='TOTAL\s+' + total_stats)

    def test_run(self):
        # ensure there is no .coverage file at the start of test
        reportfile = support_file('scenario/test_with_module/.coverage')
        try:
            os.remove(reportfile)
        except OSError:
            pass

        proc = self.runIn(
            'scenario/test_with_module',
            '-v',
            '--with-coverage',
            '--coverage=lib/'
        )
        self.assertProcOutputPattern(proc, 'lib', '\s+8\s+5\s+38%')
        self.assertTrue(os.path.exists(reportfile))

    def test_run_coverage_configs(self):
        STATS = '\s+8\s+5\s+38%\s+1, 7-10'
        TOTAL_STATS = '\s+8\s+5\s+38%\s'

        proc = self.runIn(
            'scenario/test_coverage_config/coveragerc',
            '-v',
            '--with-coverage',
            '--coverage=covered_lib_coveragerc/'
        )
        self.assertProcOutputPattern(proc, 'covered_lib_coveragerc', STATS,
                                     total_stats=TOTAL_STATS)

        proc = self.runIn(
            'scenario/test_coverage_config/nose2cfg',
            '-v',
            '--with-coverage',
            '--coverage=covered_lib_nose2cfg/'
        )
        self.assertProcOutputPattern(proc, 'covered_lib_nose2cfg', STATS,
                                     total_stats=TOTAL_STATS)

    # FIXME: figure out why this fails and remove @skip
    @unittest.skip('fails in testsuite but passes in real-world conditions')
    def test_measures_imports(self):
        proc = self.runIn(
            'scenario/coverage_of_imports',
            '-v',
            '--with-coverage',
            '--coverage=lib20171102/'
        )
        self.assertProcOutputPattern(proc, 'lib20171102',
                                     stats='\s+3\s+0\s+100%')

    def test_run_coverage_fail_under(self):
        STATS = '\s+8\s+5\s+38%\s+1, 7-10'
        TOTAL_STATS = '\s+8\s+5\s+38%\s'

        proc = self.runIn(
            'scenario/coverage_config_fail_under',
            '-v',
            '--with-coverage',
            '--coverage=covered_lib/'
        )
        self.assertProcOutputPattern(proc, 'covered_lib', STATS,
                                     total_stats=TOTAL_STATS,
                                     assert_exit_status=1)

    def test_run_coverage_fail_under2(self):
        """Check with coverage settings in config, not CLI"""
        STATS = '\s+8\s+5\s+38%\s+1, 7-10'
        TOTAL_STATS = '\s+8\s+5\s+38%\s'

        proc = self.runIn(
            'scenario/coverage_config_fail_under2',
            '-v'
        )
        self.assertProcOutputPattern(proc, 'part_covered_lib', STATS,
                                     total_stats=TOTAL_STATS,
                                     assert_exit_status=1)



import time
from nose2.tools import such


def slow_blocking_init():
    print("YEAH2")
    time.sleep(1)
    print("a second elapsed")
    time.sleep(1)
    print("a second elapsed")
    return True


class Layer1(object):

    description = "Layer1 description"

    @classmethod
    def setUp(cls):
        print("YEAH")
        it.obj = False


class Layer2(object):

    description = "Layer2 description"

    @classmethod
    def setUp(cls):
        it.obj = slow_blocking_init()


with such.A('system with a fast initial setup layer') as it:

    it.uses(Layer1)

    @it.should('not have obj initialized')
    def test():
        assert not it.obj


    with it.having('a second slow setup layer'):

        it.uses(Layer2)

        @it.should('have obj initialized')
        def test():
            assert it.obj

it.createTests(globals())



from nose2.tools import such
import logging
from .common import UniqueResource, NormalTest, NormalTestTwo
log = logging.getLogger(__name__)


with such.A('system with setup') as it:

    @it.has_setup
    def setup():
        log.info("Called setup in such test")
        it.unique_resource = UniqueResource()
        it.unique_resource.lock()

    @it.has_teardown
    def teardown():
        log.info("Called teardown in such test")
        it.unique_resource.unlock()

    @it.should('do something')
    def test(case):
        it.assertTrue(it.unique_resource.used)

it.createTests(globals())



import time

# statement run at import time should be covered
foo = 1

with open('/tmp/showme', 'a') as f:
    f.write('\n\nbeta' + str(time.time()))


raise SystemExit


# so should an ordinary function body
def func():
    return 2



# -*- coding: utf-8 -*-
import six

from nose2.tests._common import TestCase, support_file
from nose2 import util


class UtilTests(TestCase):

    def test_name_from_path(self):
        test_module = support_file('scenario/tests_in_package/pkg1/test/test_things.py')
        test_package_path = support_file('scenario/tests_in_package')
        self.assertEqual(
            util.name_from_path(test_module),
            ('pkg1.test.test_things', test_package_path)
        )

    def test_non_ascii_output(self):
        class D:
            def __init__(self):
                self.out = []
            def write(self, arg):
                self.out.append(arg)

        stream = D()
        decorated = util._WritelnDecorator(stream)
        string = six.u('\u00dcnic\u00f6de')
        decorated.write(string)
        str("".join(stream.out))



"""
Use this plugin to activate coverage report.

To use this plugin, you need to install ``nose2[coverage_plugin]``. e.g.

::

    $ pip install nose2[coverage_plugin]>=0.6.5


Then, you can enable coverage reporting with :

::

    $ nose2 --with-coverage

Or with this lines in ``unittest.cfg`` :

::

    [coverage]
    always-on = True


You can further specify coverage behaviors with a ``.coveragerc`` file, as
specified by `Coverage Config
<http://coverage.readthedocs.io/en/latest/config.html>`_.
However, when doing so you should also be aware of
`Differences From coverage`_.
"""
from __future__ import absolute_import, print_function
import six
import logging

from nose2.events import Plugin

log = logging.getLogger(__name__)


class Coverage(Plugin):
    configSection = 'coverage'
    commandLineSwitch = ('C', 'with-coverage', 'Turn on coverage reporting')

    def __init__(self):
        """Get our config and add our command line arguments."""
        # tracking var for any decision which marks the entire run as failed
        self.decided_failure = False
        # buffer for error output data
        self.error_output_buffer = six.StringIO()

        self.covSource = (self.config.as_list('coverage', []) or
                          ['.'])
        self.covReport = (self.config.as_list('coverage-report', []) or
                          ['term'])
        self.covConfig = (self.config.as_str('coverage-config', '').strip() or
                          '.coveragerc')

        group = self.session.pluginargs
        group.add_argument(
            '--coverage', action='append', default=[], metavar='PATH',
            dest='coverage_source',
            help='Measure coverage for filesystem path (multi-allowed)'
        )
        group.add_argument(
            '--coverage-report', action='append', default=[], metavar='TYPE',
            choices=['term', 'term-missing', 'annotate', 'html', 'xml'],
            dest='coverage_report',
            help='Generate selected reports, available types:'
                 ' term, term-missing, annotate, html, xml (multi-allowed)'
        )
        group.add_argument(
            '--coverage-config', action='store', default='', metavar='FILE',
            dest='coverage_config',
            help='Config file for coverage, default: .coveragerc'
        )
        self.covController = None

    def handleArgs(self, event):
        """Get our options in order command line, config file, hard coded."""
        self.covSource = event.args.coverage_source or self.covSource
        self.covReport = event.args.coverage_report or self.covReport
        self.covConfig = event.args.coverage_config or self.covConfig

    def createTests(self, event):
        """Start coverage early to catch imported modules.

        Only called if active so, safe to just start without checking flags"""
        try:
            import coverage
        except ImportError:
            print('Warning: you need to install "coverage_plugin" '
                  'extra requirements to use this plugin. '
                  'e.g. `pip install nose2[coverage_plugin]`')
            return

        if event.handled:
            log.error(
                'createTests already handled -- '
                'coverage reporting will be inaccurate')
        else:
            log.debug(
                'createTests not already handled. coverage should work')

        self.covController = coverage.Coverage(source=self.covSource,
                                               config_file=self.covConfig)
        # start immediately (don't wait until startTestRun) so that coverage
        # will pick up on things which happen at import time
        self.covController.start()

    def beforeSummaryReport(self, event):
        """Only called if active so stop coverage and produce reports."""
        if self.covController:
            self.covController.stop()

            # write to .coverage file
            # do this explicitly (instead of passing auto_data=True to
            # Coverage constructor) in order to not load an existing .coverage
            # this better imitates the behavior of invoking `coverage` from the
            # command-line, which sets `Coverage._auto_save` (triggers atexit
            # saving to this file), but not `Coverage._auto_load`
            # requesting a better fix in nedbat/coveragepy#34
            self.covController.save()

            percent_coverage = None

            if 'term' in self.covReport or 'term-missing' in self.covReport:
                # only pass `show_missing` if "term-missing" was given
                # otherwise, just allow coverage to load show_missing from
                # config
                kwargs = {}
                if 'term-missing' in self.covReport:
                    kwargs['show_missing'] = True
                percent_coverage = self.covController.report(
                    file=self.error_output_buffer, **kwargs)
            if 'annotate' in self.covReport:
                percent_coverage = self.covController.annotate()
            if 'html' in self.covReport:
                percent_coverage = self.covController.html_report()
            if 'xml' in self.covReport:
                percent_coverage = self.covController.xml_report()

            fail_under = self.covController.get_option("report:fail_under")
            if (fail_under is not None and percent_coverage is not None and
                    fail_under > percent_coverage):
                self.decided_failure = True

    def wasSuccessful(self, event):
        """Mark full test run as successful or unsuccessful"""
        if self.decided_failure:
            event.success = False

    def afterSummaryReport(self, event):
        """Reporting data is collected, failure status determined and set.
        Now print any buffered error output saved from beforeSummaryReport"""
        print(self.error_output_buffer.getvalue(), file=event.stream)



DEFAULT_PLUGINS = (
    'nose2.plugins.loader.discovery',
    'nose2.plugins.loader.testcases',
    'nose2.plugins.loader.functions',
    'nose2.plugins.loader.testclasses',
    'nose2.plugins.loader.generators',
    'nose2.plugins.loader.parameters',
    'nose2.plugins.loader.loadtests',
    'nose2.plugins.dundertest',
    'nose2.plugins.coverage',
    'nose2.plugins.result',
    'nose2.plugins.logcapture',
    'nose2.plugins.buffer',
    'nose2.plugins.failfast',
    'nose2.plugins.debugger',
    'nose2.plugins.prettyassert',
)



from nose2.tools import such
import logging
from .common import UniqueResource, NormalTest, NormalTestTwo
log = logging.getLogger(__name__)


class Layer1(object):

    description = 'Layer1'

    @classmethod
    def setUp(cls):
        log.info("Called setup in layer 1")
        it.unique_resource = UniqueResource()
        it.unique_resource.lock()

    @classmethod
    def tearDown(cls):
        log.info("Called teardown in layer 2")
        it.unique_resource.unlock()


class Layer2(object):

    description = 'Layer2'

    @classmethod
    def setUp(cls):
        log.info("Called setup in layer 2")

    @classmethod
    def tearDown(cls):
        log.info("Called teardown in layer 2")

with such.A('system with setup') as it:

    it.uses(Layer1)

    @it.should('do something')
    def test(case):
        it.assertTrue(it.unique_resource.used)

    with it.having('another setup'):

        it.uses(Layer2)

        @it.should('do something else')
        def test(case):
            it.assertTrue(it.unique_resource.used)

it.createTests(globals())



import os
import sys

from setuptools import setup

VERSION = open("nose2/_version.py").readlines()[-1].split()[-1].strip("\"'")

# TODO: potentially change author?
AUTHOR = "Jason Pellerin"
AUTHOR_EMAIL = "jpellerin+nose@gmail.com"

py_version = sys.version_info


LONG_DESCRIPTION = open(os.path.join(os.path.dirname(__file__), "README.rst")).read()

setup(
    name="nose2",
    version=VERSION,
    packages=[
        "nose2",
        "nose2.plugins",
        "nose2.plugins.loader",
        "nose2.tests",
        "nose2.tests.functional",
        "nose2.tests.unit",
        "nose2.tools",
    ],
    install_requires=[
        "six>=1.7",
        "coverage>=4.4.1",
        # mock on py2, py3.4 and py3.5
        # not just py2: py3 versions of mock don't all have the same
        # interface and this can cause issues
        'mock==2.0.0;python_version<"3.6"',
    ],
    extras_require={
        "coverage_plugin": ["coverage>=4.4.1"],
        "doc": ["Sphinx>=1.6.5", "sphinx_rtd_theme", "mock"],
    },
    entry_points={
        "console_scripts": [
            "nose2 = nose2:discover",
            "nose2-%s.%s = nose2:discover" % (py_version.major, py_version.minor),
        ]
    },
    test_suite="unittest.collector",
    # descriptive package info below
    description="unittest2 with plugins, the succesor to nose",
    long_description=LONG_DESCRIPTION,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url="https://github.com/nose-devs/nose2",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Testing",
    ],
    keywords=["unittest", "testing", "tests"],
)



import sys
import unittest
from nose2.plugins import layers
from nose2 import events, loader, session, exceptions
from nose2.tests._common import TestCase


class TestLayers(TestCase):
    tags = ['unit']

    def setUp(self):
        self.session = session.Session()
        self.loader = loader.PluggableTestLoader(session=self.session)
        self.session.testLoader = self.loader
        self.plugin = layers.Layers(session=self.session)

    def test_simple_layer_inheritance(self):
        class L1(object):
            pass

        class L2(L1):
            pass

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        class T2(unittest.TestCase):
            layer = L2

            def test(self):
                pass

        suite = unittest.TestSuite([T2('test'), T1('test')])
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        self.plugin.startTestRun(event)
        expect = [['test (nose2.tests.unit.test_layers_plugin.T1)',
                   ['test (nose2.tests.unit.test_layers_plugin.T2)']]]
        self.assertEqual(self.names(event.suite), expect)

    def test_multiple_inheritance(self):
        class L1(object):
            pass

        class L2(L1):
            pass

        class L3(L1):
            pass

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        class T2(unittest.TestCase):
            layer = L2

            def test(self):
                pass

        class T3(unittest.TestCase):
            layer = L3

            def test(self):
                pass

        suite = unittest.TestSuite([T2('test'), T1('test'), T3('test')])
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        self.plugin.startTestRun(event)
        expect = [['test (nose2.tests.unit.test_layers_plugin.T1)',
                   ['test (nose2.tests.unit.test_layers_plugin.T2)'],
                   ['test (nose2.tests.unit.test_layers_plugin.T3)']]]
        self.assertEqual(self.names(event.suite), expect)

    def test_deep_inheritance(self):
        class L1(object):
            pass

        class L2(L1):
            pass

        class L3(L1):
            pass

        class L4(L2, L1):
            pass

        class L5(L4):
            pass

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        class T2(unittest.TestCase):
            layer = L2

            def test(self):
                pass

        class T3(unittest.TestCase):
            layer = L3

            def test(self):
                pass

        class T4(unittest.TestCase):
            layer = L4

            def test(self):
                pass

        class T5(unittest.TestCase):
            layer = L5

            def test(self):
                pass

        suite = unittest.TestSuite([T2('test'), T1('test'), T3('test'),
                                    T4('test'), T5('test')])
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        self.plugin.startTestRun(event)
        expect = [['test (nose2.tests.unit.test_layers_plugin.T1)',
                   ['test (nose2.tests.unit.test_layers_plugin.T2)',
                    ['test (nose2.tests.unit.test_layers_plugin.T4)',
                     ['test (nose2.tests.unit.test_layers_plugin.T5)']]],
                   ['test (nose2.tests.unit.test_layers_plugin.T3)']]]
        self.assertEqual(self.names(event.suite), expect)

    def test_mixed_layers_no_layers(self):
        class L1(object):
            pass

        class L2(L1):
            pass

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        class T2(unittest.TestCase):
            layer = L2

            def test(self):
                pass

        class T3(unittest.TestCase):

            def test(self):
                pass

        suite = unittest.TestSuite([T2('test'), T1('test'), T3('test')])
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        self.plugin.startTestRun(event)
        expect = ['test (nose2.tests.unit.test_layers_plugin.T3)',
                  ['test (nose2.tests.unit.test_layers_plugin.T1)',
                   ['test (nose2.tests.unit.test_layers_plugin.T2)']]]
        self.assertEqual(self.names(event.suite), expect)

    def test_ordered_layers(self):
        class L1(object):
            pass

        class L2(L1):
            position = 1

        class L3(L1):
            position = 2

        class L4(L1):
            position = 3

        class L5(L2):
            position = 4

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        class T2(unittest.TestCase):
            layer = L2

            def test(self):
                pass

        class T3(unittest.TestCase):
            layer = L3

            def test(self):
                pass

        class T4(unittest.TestCase):
            layer = L4

            def test(self):
                pass

        class T5(unittest.TestCase):
            layer = L5

            def test(self):
                pass
        suite = unittest.TestSuite([T2('test'), T1('test'),
                                    T3('test'), T4('test'), T5('test')])
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        self.plugin.startTestRun(event)
        expect = [['test (nose2.tests.unit.test_layers_plugin.T1)',
                   ['test (nose2.tests.unit.test_layers_plugin.T2)',
                    ['test (nose2.tests.unit.test_layers_plugin.T5)', ]],
                   ['test (nose2.tests.unit.test_layers_plugin.T3)', ],
                   ['test (nose2.tests.unit.test_layers_plugin.T4)', ]]]
        self.assertEqual(self.names(event.suite), expect)

    def test_mixin_in_top_layer(self):

        class M1(object):
            pass

        class L1(object):
            mixins = (M1,)

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        suite = unittest.TestSuite([T1('test')])
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        self.plugin.startTestRun(event)
        expect = [  # M1
                  [  # L1
                   [  # T1
                    'test (nose2.tests.unit.test_layers_plugin.T1)']]]
        self.assertEqual(self.names(event.suite), expect)

    def test_mixin_in_inner_layer(self):

        class M1(object):
            pass

        class L1(object):
            pass

        class L2(L1):
            mixins = (M1,)

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        class T2(unittest.TestCase):
            layer = L2

            def test(self):
                pass

        suite = unittest.TestSuite([T1('test'), T2('test')])
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        self.plugin.startTestRun(event)
        expect = [  # L1
                  ['test (nose2.tests.unit.test_layers_plugin.T1)',
                   # M1
                   [  # L2
                    ['test (nose2.tests.unit.test_layers_plugin.T2)']]]]
        self.assertEqual(self.names(event.suite), expect)

    def test_mixin_inheritance(self):
        # without mixin
        # L1
        #  -> L3
        #   -> L4
        #    -> L5
        # L2
        #  -> L6
        #
        # with mixin new behavior:
        #   the mixin (L4, which comes with L3 and L1)
        #   is inserted after the parent layer (L2).
        # L2
        #  -> L1
        #   -> L3
        #    -> L4
        #     -> L6
        #     -> L5
        #
        # with mixin old behavior
        #   the mixin (L4, which comes with L3 and L1)
        #   is inserted before the parent layer (L2).
        # L1
        #  -> L3
        #   -> L4
        #    -> L2
        #     -> L6
        #    -> L5
        class L1(object):
            pass

        class L2(object):  # a mixin, doesn't share a base w/L1
            pass

        class L3(L1):
            pass

        class L4(L3):
            pass

        class L5(L4):
            pass

        class L6(L2):
            mixins = (L4,)

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        class T3(unittest.TestCase):
            layer = L3

            def test(self):
                pass

        class T4(unittest.TestCase):
            layer = L4

            def test(self):
                pass

        class T5(unittest.TestCase):
            layer = L5

            def test(self):
                pass

        class T6(unittest.TestCase):
            layer = L6

            def test(self):
                pass
        suite = unittest.TestSuite(
            [T6('test'), T1('test'), T3('test'), T4('test'), T5('test')])
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        self.plugin.startTestRun(event)
        expect = [  # L2
                  [  # L1
                   ['test (nose2.tests.unit.test_layers_plugin.T1)',  # T1
                    # L3
                    ['test (nose2.tests.unit.test_layers_plugin.T3)',  # T3
                     # L4
                     ['test (nose2.tests.unit.test_layers_plugin.T4)',  # T4
                      # L5
                      ['test (nose2.tests.unit.test_layers_plugin.T5)'],
                      # L6
                      ['test (nose2.tests.unit.test_layers_plugin.T6)']]]]]]
        self.assertEqual(self.names(event.suite), expect)

    def test_invalid_top_layer(self):

        if sys.version_info >= (3, 0):
            # in python 3, L1 will automatically have `object` has base, so
            # this test does not make sense, and will actually fail.
            return

        class L1():
            pass

        class T1(unittest.TestCase):
            layer = L1

            def test(self):
                pass

        suite = unittest.TestSuite([T1('test')])
        event = events.StartTestRunEvent(None, suite, None, 0, None)
        with self.assertRaises(exceptions.LoadTestsFailure):
            self.plugin.startTestRun(event)

    def names(self, suite):
        return [n for n in self.iternames(suite)]

    def iternames(self, suite):
        for t in suite:
            if isinstance(t, unittest.TestCase):
                if sys.version_info >= (3, 5):
                    test_module = t.__class__.__module__
                    test_class = t.__class__.__name__
                    test_method = t._testMethodName
                    yield "%s (%s.%s)" % (test_method, test_module, test_class)
                else:
                    yield str(t)
            else:
                yield [n for n in self.iternames(t)]

    def _listset(self, l):
        n = set([])
        for t in l:
            if isinstance(t, list):
                n.add(self._listset(t))
            else:
                n.add(t)
        return frozenset(n)



import sys
import os
from nose2.tests._common import FunctionalTestCase


class TestLayerHooks(FunctionalTestCase):

    @classmethod
    def setUp(cls):
        filepath = os.path.dirname(os.path.realpath(__file__))
        cls.libpath = os.path.join(filepath, 'support', 'lib')
        sys.path.append(cls.libpath)

    @classmethod
    def tearDown(cls):
        if cls.libpath in sys.path:
            sys.path.remove(cls.libpath)

    def test_simple_such(self):
        proc = self.runIn(
            'scenario/layers_hooks',
            '--plugin=nose2.plugins.layers',
            '--plugin=layer_hooks_plugin',
            'test_simple_such')
        expected = (
            "^"
            "StartLayerSetup: <class 'test_simple_such.A system:layer'>\n"
            "StopLayerSetup: <class 'test_simple_such.A system:layer'>\n"
            "StartLayerSetupTest: <class 'test_simple_such.A system:layer'>:test 0000: should do something \(test_simple_such.A system\)\n"
            "StopLayerSetupTest: <class 'test_simple_such.A system:layer'>:test 0000: should do something \(test_simple_such.A system\)\n"
            "StartLayerTeardownTest: <class 'test_simple_such.A system:layer'>:test 0000: should do something \(test_simple_such.A system\)\n"
            "StopLayerTeardownTest: <class 'test_simple_such.A system:layer'>:test 0000: should do something \(test_simple_such.A system\)\n"
            "StartLayerTeardown: <class 'test_simple_such.A system:layer'>\n"
            "StopLayerTeardown: <class 'test_simple_such.A system:layer'>\n*"
            "$")
        self.assertTestRunOutputMatches(proc, stderr='Ran 1 test')
        self.assertTestRunOutputMatches(proc, stdout=expected)
        self.assertEqual(proc.poll(), 0, proc.stderr.getvalue())

    def test_complex_such(self):
        proc = self.runIn(
            'such/',
            '--plugin=nose2.plugins.layers',
            '--plugin=layer_hooks_plugin',
            'test_such')
        expected = (
            "StartLayerSetup: <class 'test_such.A system with complex setup:layer'>\n"
            "StopLayerSetup: <class 'test_such.A system with complex setup:layer'>\n"
            "StartLayerSetupTest: <class 'test_such.A system with complex setup:layer'>:test 0000: should do something \(test_such.A system with complex setup\)\n"
            "StopLayerSetupTest: <class 'test_such.A system with complex setup:layer'>:test 0000: should do something \(test_such.A system with complex setup\)\n"
            "StartLayerTeardownTest: <class 'test_such.A system with complex setup:layer'>:test 0000: should do something \(test_such.A system with complex setup\)\n"
            "StopLayerTeardownTest: <class 'test_such.A system with complex setup:layer'>:test 0000: should do something \(test_such.A system with complex setup\)\n"
            "StartLayerSetup: <class 'test_such.having an expensive fixture:layer'>\n"
            "StopLayerSetup: <class 'test_such.having an expensive fixture:layer'>\n"
            "StartLayerSetupTest: <class 'test_such.having an expensive fixture:layer'>:test 0000: should do more things \(test_such.having an expensive fixture\)\n"
            "StopLayerSetupTest: <class 'test_such.having an expensive fixture:layer'>:test 0000: should do more things \(test_such.having an expensive fixture\)\n"
            "StartLayerTeardownTest: <class 'test_such.having an expensive fixture:layer'>:test 0000: should do more things \(test_such.having an expensive fixture\)\n"
            "StopLayerTeardownTest: <class 'test_such.having an expensive fixture:layer'>:test 0000: should do more things \(test_such.having an expensive fixture\)\n"
            "StartLayerSetup: <class 'test_such.having another precondtion:layer'>\n"
            "StopLayerSetup: <class 'test_such.having another precondtion:layer'>\n"
            "StartLayerSetupTest: <class 'test_such.having another precondtion:layer'>:test 0000: should do that not this \(test_such.having another precondtion\)\n"
            "StopLayerSetupTest: <class 'test_such.having another precondtion:layer'>:test 0000: should do that not this \(test_such.having another precondtion\)\n"
            "StartLayerTeardownTest: <class 'test_such.having another precondtion:layer'>:test 0000: should do that not this \(test_such.having another precondtion\)\n"
            "StopLayerTeardownTest: <class 'test_such.having another precondtion:layer'>:test 0000: should do that not this \(test_such.having another precondtion\)\n"
            "StartLayerSetupTest: <class 'test_such.having another precondtion:layer'>:test 0001: should do this not that \(test_such.having another precondtion\)\n"
            "StopLayerSetupTest: <class 'test_such.having another precondtion:layer'>:test 0001: should do this not that \(test_such.having another precondtion\)\n"
            "StartLayerTeardownTest: <class 'test_such.having another precondtion:layer'>:test 0001: should do this not that \(test_such.having another precondtion\)\n"
            "StopLayerTeardownTest: <class 'test_such.having another precondtion:layer'>:test 0001: should do this not that \(test_such.having another precondtion\)\n"
            "StartLayerTeardown: <class 'test_such.having another precondtion:layer'>\n"
            "StopLayerTeardown: <class 'test_such.having another precondtion:layer'>\n"
            "StartLayerSetup: <class 'test_such.SomeLayer'>\n"
            "StopLayerSetup: <class 'test_such.SomeLayer'>\n"
            "StartLayerSetup: <class 'test_such.having a different precondition:layer'>\n"
            "StopLayerSetup: <class 'test_such.having a different precondition:layer'>\n"
            "StartLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0000: should do something else \(test_such.having a different precondition\)\n"
            "StopLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0000: should do something else \(test_such.having a different precondition\)\n"
            "StartLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0000: should do something else \(test_such.having a different precondition\)\n"
            "StopLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0000: should do something else \(test_such.having a different precondition\)\n"
            "StartLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0001: should have another test \(test_such.having a different precondition\)\n"
            "StopLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0001: should have another test \(test_such.having a different precondition\)\n"
            "StartLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0001: should have another test \(test_such.having a different precondition\)\n"
            "StopLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0001: should have another test \(test_such.having a different precondition\)\n"
            "StartLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0002: should have access to an external fixture \(test_such.having a different precondition\)\n"
            "StopLayerSetupTest: <class 'test_such.having a different precondition:layer'>:test 0002: should have access to an external fixture \(test_such.having a different precondition\)\n"
            "StartLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0002: should have access to an external fixture \(test_such.having a different precondition\)\n"
            "StopLayerTeardownTest: <class 'test_such.having a different precondition:layer'>:test 0002: should have access to an external fixture \(test_such.having a different precondition\)\n"
            "StartLayerSetup: <class 'test_such.having a case inside the external fixture:layer'>\n"
            "StopLayerSetup: <class 'test_such.having a case inside the external fixture:layer'>\n"
            "StartLayerSetupTest: <class 'test_such.having a case inside the external fixture:layer'>:test 0000: should still have access to that fixture \(test_such.having a case inside the external fixture\)\n"
            "StopLayerSetupTest: <class 'test_such.having a case inside the external fixture:layer'>:test 0000: should still have access to that fixture \(test_such.having a case inside the external fixture\)\n"
            "StartLayerTeardownTest: <class 'test_such.having a case inside the external fixture:layer'>:test 0000: should still have access to that fixture \(test_such.having a case inside the external fixture\)\n"
            "StopLayerTeardownTest: <class 'test_such.having a case inside the external fixture:layer'>:test 0000: should still have access to that fixture \(test_such.having a case inside the external fixture\)\n"
            "StartLayerTeardown: <class 'test_such.having a case inside the external fixture:layer'>\n"
            "StopLayerTeardown: <class 'test_such.having a case inside the external fixture:layer'>\n"
            "StartLayerTeardown: <class 'test_such.having a different precondition:layer'>\n"
            "StopLayerTeardown: <class 'test_such.having a different precondition:layer'>\n"
            "StartLayerTeardown: <class 'test_such.SomeLayer'>\n"
            "StopLayerTeardown: <class 'test_such.SomeLayer'>\n"
            "StartLayerTeardown: <class 'test_such.having an expensive fixture:layer'>\n"
            "StopLayerTeardown: <class 'test_such.having an expensive fixture:layer'>\n"
            "StartLayerTeardown: <class 'test_such.A system with complex setup:layer'>\n"
            "StopLayerTeardown: <class 'test_such.A system with complex setup:layer'>\n*"
            "$"
        )
        self.assertTestRunOutputMatches(proc, stderr='Ran 9 tests')
        self.assertTestRunOutputMatches(proc, stdout=expected)
        self.assertEqual(proc.poll(), 0, proc.stderr.getvalue())

    def test_simple_layers(self):
        proc = self.runIn(
            'scenario/layers_hooks',
            '--plugin=nose2.plugins.layers',
            '--plugin=layer_hooks_plugin',
            'test_layers_simple')
        expected = (
            "^"
            "StartLayerSetup: <class 'test_layers_simple.Layer1'>\n"
            "StopLayerSetup: <class 'test_layers_simple.Layer1'>\n"
            "StartLayerSetupTest: <class 'test_layers_simple.Layer1'>:test_1 \(test_layers_simple.TestSimple\)\n"
            "StopLayerSetupTest: <class 'test_layers_simple.Layer1'>:test_1 \(test_layers_simple.TestSimple\)\n"
            "StartLayerTeardownTest: <class 'test_layers_simple.Layer1'>:test_1 \(test_layers_simple.TestSimple\)\n"
            "StopLayerTeardownTest: <class 'test_layers_simple.Layer1'>:test_1 \(test_layers_simple.TestSimple\)\n"
            "StartLayerSetupTest: <class 'test_layers_simple.Layer1'>:test_2 \(test_layers_simple.TestSimple\)\n"
            "StopLayerSetupTest: <class 'test_layers_simple.Layer1'>:test_2 \(test_layers_simple.TestSimple\)\n"
            "StartLayerTeardownTest: <class 'test_layers_simple.Layer1'>:test_2 \(test_layers_simple.TestSimple\)\n"
            "StopLayerTeardownTest: <class 'test_layers_simple.Layer1'>:test_2 \(test_layers_simple.TestSimple\)\n"
            "StartLayerTeardown: <class 'test_layers_simple.Layer1'>\n"
            "StopLayerTeardown: <class 'test_layers_simple.Layer1'>\n*"
            "$")
        self.assertTestRunOutputMatches(proc, stderr='Ran 2 tests')
        self.assertTestRunOutputMatches(proc, stdout=expected)
        self.assertEqual(proc.poll(), 0, proc.stderr.getvalue())



import nose2
import unittest
import logging

log = logging.getLogger(__name__)


class Test(unittest.TestCase):

    def test_logging_config(self):
        log.debug("foo")
        log.info("bar")
        assert False


if __name__ == '__main__':
    nose2.main()
